package net.locusworks.worldcities.csv;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVRecord;

import net.locusworks.worldcities.entities.City;
import net.locusworks.worldcities.entities.Location;

public class CityFileReader {
  
  private static final String[] HEADERS = {"city","city_ascii","lat","lng","country","iso2","iso3","admin_name","capital","population","id"};

  private static List<City> worldCities;
  
  public static List<City> readCities() throws IOException {
    if (worldCities != null && worldCities.size() > 0) return worldCities; 
    
    InputStream is = CityFileReader.class.getClassLoader().getResourceAsStream("worldcities.csv");
    
    Iterable<CSVRecord> records = CSVFormat.DEFAULT
        .withHeader(HEADERS)
        .withFirstRecordAsHeader()
        .parse(new InputStreamReader(is));
    
    worldCities =  StreamSupport.stream(records.spliterator(), false)
        .map(r -> toCity(r))
        .collect(Collectors.toList());
    
    return worldCities;
  }
  
  private static City toCity(CSVRecord record) {
    String city = record.get("city_ascii");
    float lat = Float.valueOf(record.get("lat"));
    float lng = Float.valueOf(record.get("lng"));;
    String country = record.get("country");
    String iso2 = record.get("iso2");
    String iso3 = record.get("iso3");
    long id = Long.valueOf(record.get("id"));
    
    return new City(id, city, country, iso2, iso3, new Location(lat, lng));
    
  }

}
